@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -31,6 +31,7 @@ extensions = [
     "sphinx.ext.viewcode",
     "sphinx.ext.intersphinx",
     "hoverxref.extension",
+    "sphinx_selective_exclude.eager_only",
 ]
 
 templates_path = ["_templates"]
@@ -53,6 +54,12 @@ with open(
     version = _d["__version__"]
     release = _d["__version__"]
 
+
+def setup(app):
+    if os.path.isfile(os.path.join(os.path.dirname(__file__), "..", "RELEASE.rst")):
+        app.tags.add("has_release_file")
+
+
 language = None
 
 exclude_patterns = ["_build"]
@@ -71,6 +78,7 @@ intersphinx_mapping = {
     "pandas": ("https://pandas.pydata.org/pandas-docs/stable/", None),
     "pytest": ("https://docs.pytest.org/en/stable/", None),
     "django": ("https://django.readthedocs.io/en/stable/", None),
+    "dateutil": ("https://dateutil.readthedocs.io/en/stable/", None),
     "redis": ("https://redis-py.readthedocs.io/en/stable/", None),
     "attrs": ("https://www.attrs.org/en/stable/", None),
 }
