@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,14 +13,11 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import re
 
 import pytest
 
-import hypothesis.strategies as st
-from hypothesis import Verbosity, given, settings
+from hypothesis import Verbosity, given, settings, strategies as st
 from tests.common.utils import capture_out
 
 
@@ -38,7 +33,7 @@ def test_reports_passes():
 
     value = out.getvalue()
 
-    assert "adaptive_example_deletion" in value
+    assert "minimize_individual_blocks" in value
     assert "calls" in value
     assert "shrinks" in value
 
@@ -49,4 +44,4 @@ def test_reports_passes():
         if m is not None and int(m.group(2)) != 0:
             break
     else:
-        assert False, value
+        pytest.xfail(reason="Sometimes the first failure is 10, and cannot shrink.")
