@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,6 +15,7 @@
 
 import collections
 import decimal
+import enum
 import fractions
 import math
 from datetime import date, datetime, time, timedelta
@@ -137,6 +138,7 @@ def fn_ktest(*fnkwargs):
     (ds.floats, {"min_value": 0.0, "max_value": 1.0, "allow_infinity": True}),
     (ds.floats, {"min_value": math.inf, "allow_infinity": False}),
     (ds.floats, {"max_value": -math.inf, "allow_infinity": False}),
+    (ds.complex_numbers, {"min_magnitude": None}),
     (ds.complex_numbers, {"min_magnitude": math.nan}),
     (ds.complex_numbers, {"max_magnitude": math.nan}),
     (ds.complex_numbers, {"max_magnitude": complex(1, 2)}),
@@ -182,6 +184,8 @@ def fn_ktest(*fnkwargs):
     (ds.ip_addresses, {"v": 6, "network": "127.0.0.0/8"}),
     (ds.ip_addresses, {"network": b"127.0.0.0/8"}),  # only unicode strings are valid
     (ds.ip_addresses, {"network": b"::/64"}),
+    (ds.randoms, {"use_true_random": "False"}),
+    (ds.randoms, {"note_method_calls": "True"}),
 )
 def test_validates_keyword_arguments(fn, kwargs):
     with pytest.raises(InvalidArgument):
@@ -494,3 +498,31 @@ def test_ipaddress_from_network_is_always_in_network(data, network):
     ip = data.draw(ds.ip_addresses(network=network), label="address")
     assert ip in network
     assert ip.version == network.version
+
+
+class AnEnum(enum.Enum):
+    a = 1
+
+
+def requires_arg(value):
+    """Similar to the enum.Enum.__call__ method."""
+
+
+@given(ds.data())
+def test_builds_error_messages(data):
+    # If we call them directly, we get a simple TypeError in both cases
+    with pytest.raises(TypeError):
+        requires_arg()
+    with pytest.raises(TypeError):
+        AnEnum()
+    # But we have an improved error message if you try to build an Enum
+    assert issubclass(InvalidArgument, TypeError)  # it's a valid substitution
+    with pytest.raises(TypeError):  # which only applies to enums
+        data.draw(ds.builds(requires_arg))
+    with pytest.raises(
+        InvalidArgument,
+        match=r".* try using sampled_from\(.+\) instead of builds\(.+\)",
+    ):
+        data.draw(ds.builds(AnEnum))
+    # and sampled_from() does in fact work
+    data.draw(ds.sampled_from(AnEnum))
