@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -17,10 +17,13 @@ import abc
 import collections
 import datetime
 import enum
+import inspect
 import io
+import re
 import string
 import sys
 import typing
+from inspect import signature
 from numbers import Real
 
 import pytest
@@ -46,7 +49,7 @@ generics = sorted(
 xfail_on_39 = () if sys.version_info[:2] < (3, 9) else pytest.mark.xfail
 
 
-@pytest.mark.parametrize("typ", generics)
+@pytest.mark.parametrize("typ", generics, ids=repr)
 @settings(
     suppress_health_check=[HealthCheck.too_slow, HealthCheck.filter_too_much],
     database=None,
@@ -90,6 +93,24 @@ def test_typing_Type_Union(ex):
     assert ex in (str, list)
 
 
+@pytest.mark.parametrize(
+    "typ",
+    [
+        collections.abc.ByteString,
+        # These are nonexistent or exist-but-are-not-types on Python 3.6
+        typing.Match if sys.version_info[:2] >= (3, 7) else int,
+        typing.Pattern if sys.version_info[:2] >= (3, 7) else int,
+        getattr(re, "Match", int),
+        getattr(re, "Pattern", int),
+    ],
+    ids=repr,
+)
+@given(data=st.data())
+def test_rare_types(data, typ):
+    ex = data.draw(from_type(typ))
+    assert isinstance(ex, typ)
+
+
 class Elem:
     pass
 
@@ -112,6 +133,8 @@ class Elem:
         (typing.Mapping[Elem, None], typing.Mapping),
         (typing.Container[Elem], typing.Container),
         (typing.NamedTuple("A_NamedTuple", (("elem", Elem),)), tuple),
+        (typing.Counter[Elem], typing.Counter),
+        (typing.Deque[Elem], typing.Deque),
     ],
     ids=repr,
 )
@@ -124,9 +147,30 @@ def test_specialised_collection_types(data, typ, coll_type):
     assume(instances)  # non-empty collections without calling len(iterator)
 
 
-@given(from_type(typing.DefaultDict[int, Elem]).filter(len))
-def test_defaultdict_values_type(ex):
-    assert all(isinstance(elem, Elem) for elem in ex.values())
+class ElemValue:
+    pass
+
+
+@pytest.mark.parametrize(
+    "typ,coll_type",
+    [
+        (typing.ChainMap[Elem, ElemValue], typing.ChainMap),
+        (typing.DefaultDict[Elem, ElemValue], typing.DefaultDict),
+    ]
+    + (
+        [(typing.OrderedDict[Elem, ElemValue], typing.OrderedDict)]
+        if hasattr(typing, "OrderedDict")  # Python 3.7.2 and later
+        else []
+    ),
+    ids=repr,
+)
+@given(data=st.data())
+def test_specialised_mapping_types(data, typ, coll_type):
+    ex = data.draw(from_type(typ).filter(len))
+    assert isinstance(ex, coll_type)
+    instances = [isinstance(elem, Elem) for elem in ex]
+    assert all(instances)
+    assert all(isinstance(elem, ElemValue) for elem in ex.values())
 
 
 @given(from_type(typing.ItemsView[Elem, Elem]).filter(len))
@@ -137,6 +181,27 @@ def test_ItemsView(ex):
     assert all(all(isinstance(e, Elem) for e in elem) for elem in ex)
 
 
+@pytest.mark.skipif(sys.version_info[:2] == (3, 6), reason="not a type on py36")
+@pytest.mark.parametrize("generic", [typing.Match, typing.Pattern])
+@pytest.mark.parametrize("typ", [bytes, str])
+@given(data=st.data())
+def test_regex_types(data, generic, typ):
+    x = data.draw(from_type(generic[typ]))
+    assert isinstance(x[0] if generic is typing.Match else x.pattern, typ)
+
+
+@given(x=infer)
+def test_Generator(x: typing.Generator[Elem, None, ElemValue]):
+    assert isinstance(x, typing.Generator)
+    try:
+        while True:
+            e = next(x)
+            assert isinstance(e, Elem)
+            x.send(None)  # The generators we create don't check the send type
+    except StopIteration as stop:
+        assert isinstance(stop.value, ElemValue)
+
+
 def test_Optional_minimises_to_None():
     assert minimal(from_type(typing.Optional[int]), lambda ex: True) is None
 
@@ -606,7 +671,7 @@ def test_can_cast():
 @pytest.mark.parametrize("type_", [datetime.timezone, datetime.tzinfo])
 def test_timezone_lookup(type_):
     assert issubclass(type_, datetime.tzinfo)
-    assert_all_examples(st.from_type(type_), lambda t: isinstance(t, datetime.timezone))
+    assert_all_examples(st.from_type(type_), lambda t: isinstance(t, type_))
 
 
 @pytest.mark.parametrize(
@@ -655,3 +720,86 @@ class TreeForwardRefs(typing.NamedTuple):
 @given(st.builds(TreeForwardRefs))
 def test_resolves_forward_references_outside_annotations(t):
     assert isinstance(t, TreeForwardRefs)
+
+
+def constructor(a: str = None):
+    pass
+
+
+class WithOptionalInSignature:
+    __signature__ = inspect.signature(constructor)
+    __annotations__ = typing.get_type_hints(constructor)
+
+    def __init__(self, **kwargs):
+        assert set(kwargs) == {"a"}
+        self.a = kwargs["a"]
+
+
+def test_compat_get_type_hints_aware_of_None_default():
+    # Regression test for https://github.com/HypothesisWorks/hypothesis/issues/2648
+    strategy = st.builds(WithOptionalInSignature, a=infer)
+    find_any(strategy, lambda x: x.a is None)
+    find_any(strategy, lambda x: x.a is not None)
+
+    assert typing.get_type_hints(constructor)["a"] == typing.Optional[str]
+    assert inspect.signature(constructor).parameters["a"].annotation == str
+
+
+_ValueType = typing.TypeVar("_ValueType")
+
+
+class Wrapper(typing.Generic[_ValueType]):
+    _inner_value: _ValueType
+
+    def __init__(self, inner_value: _ValueType) -> None:
+        self._inner_value = inner_value
+
+
+@given(st.builds(Wrapper))
+def test_issue_2603_regression(built):
+    """It was impossible to build annotated classes with constructors."""
+    assert isinstance(built, Wrapper)
+
+
+class AnnotatedConstructor(typing.Generic[_ValueType]):
+    value: _ValueType  # the same name we have in `__init__`
+
+    def __init__(self, value: int) -> None:
+        """By this example we show, that ``int`` is more important than ``_ValueType``."""
+        assert isinstance(value, int)
+
+
+@given(st.data())
+def test_constructor_is_more_important(data):
+    """Constructor types should take precedence over all other annotations."""
+    data.draw(st.builds(AnnotatedConstructor))
+
+
+def use_signature(self, value: str) -> None:
+    ...
+
+
+class AnnotatedConstructorWithSignature(typing.Generic[_ValueType]):
+    value: _ValueType  # the same name we have in `__init__`
+
+    __signature__ = signature(use_signature)
+
+    def __init__(self, value: int) -> None:
+        """By this example we show, that ``__signature__`` is the most important source."""
+        assert isinstance(value, str)
+
+
+@given(st.data())
+def test_signature_is_the_most_important_source(data):
+    """Signature types should take precedence over all other annotations."""
+    data.draw(st.builds(AnnotatedConstructorWithSignature))
+
+
+class AnnotatedAndDefault:
+    def __init__(self, foo: bool = None):
+        self.foo = foo
+
+
+def test_from_type_can_be_default_or_annotation():
+    find_any(st.from_type(AnnotatedAndDefault), lambda x: x.foo is None)
+    find_any(st.from_type(AnnotatedAndDefault), lambda x: isinstance(x.foo, bool))
