@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,8 +13,6 @@
 #
 # END HEADER
 
-# coding: utf-8
-
 """This file originates in the IPython project and is made use of under the
 following licensing terms:
 
@@ -56,31 +52,19 @@ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """
 
-from __future__ import absolute_import, division, print_function
-
 import re
 from collections import Counter, OrderedDict, defaultdict, deque
+from io import StringIO
 
 import pytest
 
-from hypothesis.internal.compat import PY3, PYPY, a_good_encoding
+from hypothesis.internal.compat import PYPY
 from hypothesis.vendor import pretty
 from tests.common.utils import capture_out
 
-py2_only = pytest.mark.skipif(PY3, reason="This test only runs on python 2")
 
-if PY3:
-    from io import StringIO
-
-    def unicode_to_str(x, encoding=None):
-        return x
-
-
-else:
-    from StringIO import StringIO
-
-    def unicode_to_str(x, encoding=None):
-        return x.encode(encoding or a_good_encoding())
+def unicode_to_str(x, encoding=None):
+    return x
 
 
 def assert_equal(x, y):
@@ -106,7 +90,7 @@ def skip_without(mod):
 assert_raises = pytest.raises
 
 
-class MyList(object):
+class MyList:
     def __init__(self, content):
         self.content = content
 
@@ -129,12 +113,12 @@ class MyDict(dict):
         p.text("MyDict(...)")
 
 
-class MyObj(object):
+class MyObj:
     def somemethod(self):
         pass
 
 
-class Dummy1(object):
+class Dummy1:
     def _repr_pretty_(self, p, cycle):
         p.text("Dummy1(...)")
 
@@ -143,14 +127,14 @@ class Dummy2(Dummy1):
     _repr_pretty_ = None
 
 
-class NoModule(object):
+class NoModule:
     pass
 
 
 NoModule.__module__ = None
 
 
-class Breaking(object):
+class Breaking:
     def _repr_pretty_(self, p, cycle):
         with p.group(4, "TG: ", ":"):
             p.text("Breaking(")
@@ -158,18 +142,18 @@ class Breaking(object):
             p.text(")")
 
 
-class BreakingRepr(object):
+class BreakingRepr:
     def __repr__(self):
         return "Breaking(\n)"
 
 
-class BreakingReprParent(object):
+class BreakingReprParent:
     def _repr_pretty_(self, p, cycle):
         with p.group(4, "TG: ", ":"):
             p.pretty(BreakingRepr())
 
 
-class BadRepr(object):
+class BadRepr:
     def __repr__(self):
         return 1 / 0
 
@@ -323,7 +307,7 @@ class BadException(Exception):
         return -1
 
 
-class ReallyBadRepr(object):
+class ReallyBadRepr:
     __module__ = 1
 
     @property
@@ -339,7 +323,7 @@ def test_really_bad_repr():
         pretty.pretty(ReallyBadRepr())
 
 
-class SA(object):
+class SA:
     pass
 
 
@@ -417,10 +401,10 @@ def test_metaclass_repr():
 
 
 def test_unicode_repr():
-    u = u"üniçodé"
+    u = "üniçodé"
     ustr = unicode_to_str(u)
 
-    class C(object):
+    class C:
         def __repr__(self):
             return ustr
 
@@ -428,7 +412,7 @@ def test_unicode_repr():
     p = pretty.pretty(c)
     assert_equal(p, u)
     p = pretty.pretty([c])
-    assert_equal(p, u"[%s]" % u)
+    assert_equal(p, "[%s]" % u)
 
 
 def test_basic_class():
@@ -450,83 +434,6 @@ def test_basic_class():
     assert_true(type_pprint_wrapper.called)
 
 
-# This is only run on Python 2 because in Python 3 the language prevents you
-# from setting a non-unicode value for __qualname__ on a metaclass, and it
-# doesn't respect the descriptor protocol if you subclass unicode and implement
-# __get__.
-@py2_only
-def test_fallback_to__name__on_type():
-    # Test that we correctly repr types that have non-string values for
-    # __qualname__ by falling back to __name__
-
-    class Type(object):
-        __qualname__ = 5
-
-    # Test repring of the type.
-    stream = StringIO()
-    printer = pretty.RepresentationPrinter(stream)
-
-    printer.pretty(Type)
-    printer.flush()
-    output = stream.getvalue()
-
-    # If __qualname__ is malformed, we should fall back to __name__.
-    expected = ".".join([__name__, Type.__name__])
-    assert_equal(output, expected)
-
-    # Clear stream buffer.
-    stream.buf = ""
-
-    # Test repring of an instance of the type.
-    instance = Type()
-    printer.pretty(instance)
-    printer.flush()
-    output = stream.getvalue()
-
-    # Should look like:
-    # <IPython.lib.tests.test_pretty.Type at 0x7f7658ae07d0>
-    prefix = "<" + ".".join([__name__, Type.__name__]) + " at 0x"
-    assert_true(output.startswith(prefix))
-
-
-@py2_only
-def test_fail_gracefully_on_bogus__qualname__and__name__():
-    # Test that we correctly repr types that have non-string values for both
-    # __qualname__ and __name__
-
-    class Meta(type):
-        __name__ = 5
-
-    class Type(object):
-        __metaclass__ = Meta  # noqa
-        __qualname__ = 5
-
-    stream = StringIO()
-    printer = pretty.RepresentationPrinter(stream)
-
-    printer.pretty(Type)
-    printer.flush()
-    output = stream.getvalue()
-
-    # If we can't find __name__ or __qualname__ just use a sentinel string.
-    expected = ".".join([__name__, "<unknown type>"])
-    assert_equal(output, expected)
-
-    # Clear stream buffer.
-    stream.buf = ""
-
-    # Test repring of an instance of the type.
-    instance = Type()
-    printer.pretty(instance)
-    printer.flush()
-    output = stream.getvalue()
-
-    # Should look like:
-    # <IPython.lib.tests.test_pretty.<unknown type> at 0x7f7658ae07d0>
-    prefix = "<" + ".".join([__name__, "<unknown type>"]) + " at 0x"
-    assert_true(output.startswith(prefix))
-
-
 def test_collections_defaultdict():
     # Create defaultdicts with cycles
     a = defaultdict()
@@ -550,7 +457,7 @@ def test_collections_defaultdict():
         assert_equal(pretty.pretty(obj), expected)
 
 
-@pytest.mark.skipif(PY3 and PYPY, reason="slightly different on PyPy3")
+@pytest.mark.skipif(PYPY, reason="slightly different on PyPy3")
 def test_collections_ordereddict():
     # Create OrderedDict with cycle
     a = OrderedDict()
@@ -638,7 +545,7 @@ def test_cyclic_dequeue():
     assert pretty.pretty(x) == "deque([deque(...)])"
 
 
-class HashItAnyway(object):
+class HashItAnyway:
     def __init__(self, value):
         self.value = value
 
@@ -713,13 +620,13 @@ def test_re_evals():
         re.compile(r"hi"),
         re.compile(r"b\nc", re.MULTILINE),
         re.compile(br"hi", 0),
-        re.compile(u"foo", re.MULTILINE | re.UNICODE),
+        re.compile("foo", re.MULTILINE | re.UNICODE),
     ]:
         r2 = eval(pretty.pretty(r), globals())
         assert r.pattern == r2.pattern and r.flags == r2.flags
 
 
-class CustomStuff(object):
+class CustomStuff:
     def __init__(self):
         self.hi = 1
         self.bye = "fish"
@@ -749,13 +656,13 @@ def test_pretty_function():
 
 def test_empty_printer():
     printer = pretty.RepresentationPrinter(
-        pretty.CUnicodeIO(),
+        StringIO(),
         singleton_pprinters={},
         type_pprinters={int: pretty._repr_pprint, list: pretty._repr_pprint},
         deferred_pprinters={},
     )
     printer.pretty([1, 2, 3])
-    assert printer.output.getvalue() == u"[1, 2, 3]"
+    assert printer.output.getvalue() == "[1, 2, 3]"
 
 
 def test_breakable_at_group_boundary():
