@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,16 +13,17 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import sys
 from copy import deepcopy
+from datetime import time
 from functools import partial
+from inspect import FullArgSpec, getfullargspec
+from unittest.mock import MagicMock, Mock, NonCallableMagicMock, NonCallableMock
 
 import pytest
 
-import hypothesis.internal.reflection as reflection
-from hypothesis.internal.compat import PY2, PY3, FullArgSpec, getfullargspec
+from hypothesis import strategies as st
+from hypothesis.internal import reflection
 from hypothesis.internal.reflection import (
     arg_string,
     convert_keyword_arguments,
@@ -41,11 +40,6 @@ from hypothesis.internal.reflection import (
 )
 from tests.common.utils import raises
 
-try:
-    from unittest.mock import MagicMock, Mock, NonCallableMagicMock, NonCallableMock
-except ImportError:
-    from mock import MagicMock, Mock, NonCallableMagicMock, NonCallableMock
-
 
 def do_conversion_test(f, args, kwargs):
     result = f(*args, **kwargs)
@@ -189,7 +183,7 @@ def test_names_of_functions_are_pretty():
     )
 
 
-class Foo(object):
+class Foo:
     @classmethod
     def bar(cls):
         pass
@@ -296,7 +290,7 @@ def test_arg_string_does_not_include_unprovided_defaults():
     assert arg_string(foo, (1,), {"b": 1, "d": 11}) == "a=1, b=1, d=11"
 
 
-class A(object):
+class A:
     def f(self):
         pass
 
@@ -520,7 +514,7 @@ def test_define_function_signature_validates_function_name():
         )
 
 
-class Container(object):
+class Container:
     def funcy(self):
         pass
 
@@ -587,12 +581,12 @@ def test_can_proxy_lambdas(func, args, expected):
     assert wrapped(*args) == expected
 
 
-class Snowman(object):
+class Snowman:
     def __repr__(self):
         return "☃"
 
 
-class BittySnowman(object):
+class BittySnowman:
     def __repr__(self):
         return "☃"
 
@@ -605,7 +599,7 @@ def test_can_handle_unicode_repr():
     assert arg_string(foo, [], {"x": Snowman()}) == "x=☃"
 
 
-class NoRepr(object):
+class NoRepr:
     pass
 
 
@@ -625,16 +619,6 @@ def test_can_handle_repr_of_none():
     assert arg_string(foo, [], {"x": None}) == "x=None"
 
 
-if not PY3:
-
-    def test_can_handle_non_unicode_repr_containing_non_ascii():
-        def foo(x):
-            pass
-
-        assert arg_string(foo, [BittySnowman()], {}) == "x=☃"
-        assert arg_string(foo, [], {"x": BittySnowman()}) == "x=☃"
-
-
 def test_kwargs_appear_in_arg_string():
     def varargs(*args, **kwargs):
         pass
@@ -659,7 +643,7 @@ def test_is_mock_with_positive_cases():
     assert is_mock(NonCallableMagicMock())
 
 
-class Target(object):
+class Target:
     def __init__(self, a, b):
         pass
 
@@ -693,7 +677,6 @@ def test_can_handle_unicode_identifier_in_same_line_as_lambda_def():
     assert get_pretty_function_description(is_str_pi) == "lambda x: x == pi"
 
 
-@pytest.mark.skipif(PY2, reason="detect_encoding does not exist in Python 2")
 def test_can_render_lambda_with_no_encoding(monkeypatch):
     is_positive = lambda x: x > 0
 
@@ -705,7 +688,6 @@ def test_can_render_lambda_with_no_encoding(monkeypatch):
     assert get_pretty_function_description(is_positive) == "lambda x: x > 0"
 
 
-@pytest.mark.skipif(PY2, reason="detect_encoding does not exist in Python 2")
 def test_does_not_crash_on_utf8_lambda_without_encoding(monkeypatch):
     # Monkey-patching out the `detect_encoding` method here means
     # that our reflection can't detect the encoding of the source file, and
@@ -713,3 +695,13 @@ def test_does_not_crash_on_utf8_lambda_without_encoding(monkeypatch):
 
     monkeypatch.setattr(reflection, "detect_encoding", None)
     assert get_pretty_function_description(is_str_pi) == "lambda x: <unknown>"
+
+
+def test_too_many_posargs_fails():
+    with pytest.raises(TypeError):
+        st.times(time.min, time.max, st.none(), st.none()).validate()
+
+
+def test_overlapping_posarg_kwarg_fails():
+    with pytest.raises(TypeError):
+        st.times(time.min, time.max, st.none(), timezones=st.just(None)).validate()
