@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -453,3 +453,12 @@ def test_fullmatch_must_be_bool():
 
 def test_issue_1786_regression():
     st.from_regex(re.compile("\\\\", flags=re.IGNORECASE)).validate()
+
+
+def test_sets_allow_multichar_output_in_ignorecase_mode():
+    # CharacterBuilder.strategy includes logic to add multi-character strings
+    # via sampled_from(), if any are whitelisted as matching.  See issue #2657.
+    find_any(
+        st.from_regex(re.compile("[\u0130_]", re.IGNORECASE)),
+        lambda s: len(s) > 1,
+    )
