@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -30,7 +30,11 @@ from hypothesis._settings import (
     settings,
 )
 from hypothesis.database import ExampleDatabase
-from hypothesis.errors import InvalidArgument, InvalidState
+from hypothesis.errors import (
+    HypothesisDeprecationWarning,
+    InvalidArgument,
+    InvalidState,
+)
 from hypothesis.stateful import RuleBasedStateMachine, rule
 from hypothesis.utils.conventions import not_set
 from tests.common.utils import counts_calls, fails_with
@@ -87,19 +91,6 @@ def test_can_repeatedly_push_the_same_thing():
     assert settings().max_examples == original_default
 
 
-def test_cannot_create_settings_with_invalid_options():
-    with pytest.raises(InvalidArgument):
-        settings(a_setting_with_limited_options="spoon")
-
-
-def test_cannot_register_with_parent_and_settings_args():
-    with pytest.raises(InvalidArgument):
-        settings.register_profile(
-            "conflicted", settings.default, settings=settings.default
-        )
-    assert "conflicted" not in settings._profiles
-
-
 def test_can_set_verbosity():
     settings(verbosity=Verbosity.quiet)
     settings(verbosity=Verbosity.normal)
@@ -200,11 +191,12 @@ def test_can_have_none_database():
 
 
 @pytest.mark.parametrize("db", [None, ExampleDatabase(":memory:")])
-def test_database_type_must_be_ExampleDatabase(db):
+@pytest.mark.parametrize("bad_db", [":memory:", ".hypothesis/examples"])
+def test_database_type_must_be_ExampleDatabase(db, bad_db):
     with local_settings(settings(database=db)):
         settings_property_db = settings.database
         with pytest.raises(InvalidArgument):
-            settings(database=".hypothesis/examples")
+            settings(database=bad_db)
         assert settings.database is settings_property_db
 
 
@@ -439,11 +431,6 @@ def test_assigning_to_settings_attribute_on_state_machine_raises_error():
     state_machine_instance.settings = "any value"
 
 
-def test_can_not_set_timeout_to_time():
-    with pytest.raises(InvalidArgument):
-        settings(timeout=60)
-
-
 def test_derandomise_with_explicit_database_is_invalid():
     with pytest.raises(InvalidArgument):
         settings(derandomize=True, database=ExampleDatabase(":memory:"))
@@ -454,7 +441,6 @@ def test_derandomise_with_explicit_database_is_invalid():
     [
         {"max_examples": -1},
         {"max_examples": 2.5},
-        {"buffer_size": -1},
         {"stateful_step_count": -1},
         {"stateful_step_count": 2.5},
         {"deadline": -1},
@@ -488,7 +474,15 @@ def test_show_changed():
 
 def test_note_deprecation_checks_date():
     with pytest.warns(None) as rec:
-        note_deprecation("This is bad", since="RELEASEDAY")
+        note_deprecation("This is bad", since="RELEASEDAY", has_codemod=False)
     assert len(rec) == 1
     with pytest.raises(AssertionError):
-        note_deprecation("This is way too old", since="1999-12-31")
+        note_deprecation("This is way too old", since="1999-12-31", has_codemod=False)
+
+
+def test_note_deprecation_checks_has_codemod():
+    with pytest.warns(
+        HypothesisDeprecationWarning,
+        match="The `hypothesis codemod` command-line tool",
+    ):
+        note_deprecation("This is bad", since="2021-01-01", has_codemod=True)
