@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -29,7 +29,7 @@ from hypothesis.strategies._internal import types
 from hypothesis.strategies._internal.core import _strategies
 from hypothesis.strategies._internal.types import _global_type_lookup
 from tests.common.debug import assert_all_examples, find_any
-from tests.common.utils import checks_deprecated_behaviour, fails_with, temp_registered
+from tests.common.utils import fails_with, temp_registered
 
 # Build a set of all types output by core strategies
 blacklist = [
@@ -43,6 +43,8 @@ blacklist = [
     "randoms",
     "runner",
     "sampled_from",
+    "timezone_keys",
+    "timezones",
 ]
 types_with_core_strat = set()
 for thing in (
@@ -244,21 +246,10 @@ _skip_callables_mark = pytest.mark.skipif(sys.version_info[:2] < (3, 7), reason=
     ids=repr,
 )
 @pytest.mark.parametrize("strategy", [st.none(), lambda _: st.none()])
-@checks_deprecated_behaviour
 def test_generic_origin_with_type_args(generic, strategy):
-    try:
-        # Registering a generic type with args is deprecated
+    with pytest.raises(InvalidArgument):
         st.register_type_strategy(generic, strategy)
-        assert generic not in types._global_type_lookup
-        assert generic.__origin__ in types._global_type_lookup
-        # But trying to register another strategy does, since that could be
-        # a symptom of trying multiple registrations for different args
-        with pytest.raises(InvalidArgument):
-            st.register_type_strategy(generic, strategy)
-    finally:
-        st.from_type.__clear_cache()
-        for x in (generic, generic.__origin__):
-            types._global_type_lookup.pop(x, None)
+    assert generic not in types._global_type_lookup
 
 
 @pytest.mark.parametrize(
