@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -101,11 +101,16 @@ def test_time_bounds_must_be_naive(name, val):
         times(**{name: val}).validate()
 
 
-def test_can_trigger_error_in_draw_near_max_date():
+@pytest.mark.parametrize(
+    "bound",
+    [
+        {"min_value": dt.datetime.max - dt.timedelta(days=3)},
+        {"max_value": dt.datetime.min + dt.timedelta(days=3)},
+    ],
+)
+def test_can_trigger_error_in_draw_near_boundary(bound):
     assert_can_trigger_event(
-        datetimes(
-            min_value=dt.datetime.max - dt.timedelta(days=3), timezones=timezones()
-        ),
+        datetimes(**bound, timezones=timezones()),
         lambda event: "Failed to draw a datetime" in event,
     )
 
