@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,79 +13,27 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import sys
 from functools import reduce
+from itertools import zip_longest
 
 import numpy as np
 import pytest
-import six
 
-import hypothesis.extra.numpy as nps
-import hypothesis.strategies as st
-from hypothesis import HealthCheck, assume, given, note, settings
+from hypothesis import HealthCheck, assume, given, note, settings, strategies as st
 from hypothesis.errors import InvalidArgument, Unsatisfiable
-from hypothesis.internal.compat import PY2, binary_type, text_type
-from hypothesis.searchstrategy import SearchStrategy
+from hypothesis.extra import numpy as nps
 from tests.common.debug import find_any, minimal
-from tests.common.utils import checks_deprecated_behaviour, fails_with, flaky
-
-if PY2:
-    from itertools import izip_longest as zip_longest
-else:
-    from itertools import zip_longest
-
+from tests.common.utils import fails_with, flaky
 
 ANY_SHAPE = nps.array_shapes(min_dims=0, max_dims=32, min_side=0, max_side=32)
 ANY_NONZERO_SHAPE = nps.array_shapes(min_dims=0, max_dims=32, min_side=1, max_side=32)
-STANDARD_TYPES = list(
-    map(
-        np.dtype,
-        [
-            u"int8",
-            u"int16",
-            u"int32",
-            u"int64",
-            u"uint8",
-            u"uint16",
-            u"uint32",
-            u"uint64",
-            u"float",
-            u"float16",
-            u"float32",
-            u"float64",
-            u"complex64",
-            u"complex128",
-            u"datetime64",
-            u"timedelta64",
-            bool,
-            text_type,
-            binary_type,
-        ],
-    )
-)
-
-
-@given(nps.nested_dtypes())
-def test_strategies_for_standard_dtypes_have_reusable_values(dtype):
-    assert nps.from_dtype(dtype).has_reusable_values
-
-
-@pytest.mark.parametrize(u"t", STANDARD_TYPES)
-def test_produces_instances(t):
-    @given(nps.from_dtype(t))
-    def test_is_t(x):
-        assert isinstance(x, t.type)
-        assert x.dtype.kind == t.kind
-
-    test_is_t()
 
 
 @given(nps.arrays(float, ()))
 def test_empty_dimensions_are_arrays(x):
     assert isinstance(x, np.ndarray)
-    assert x.dtype.kind == u"f"
+    assert x.dtype.kind == "f"
 
 
 @given(nps.arrays(float, (1, 0, 1)))
@@ -95,21 +41,11 @@ def test_can_handle_zero_dimensions(x):
     assert x.shape == (1, 0, 1)
 
 
-@given(nps.arrays(u"uint32", (5, 5)))
+@given(nps.arrays("uint32", (5, 5)))
 def test_generates_unsigned_ints(x):
     assert (x >= 0).all()
 
 
-@given(st.data())
-def test_can_handle_long_shapes(data):
-    """We can eliminate this test once we drop Py2 support."""
-    for tt in six.integer_types:
-        X = data.draw(nps.arrays(float, (tt(5),)))
-        assert X.shape == (5,)
-        X = data.draw(nps.arrays(float, (tt(5), tt(5))))
-        assert X.shape == (5, 5)
-
-
 @given(nps.arrays(int, (1,)))
 def test_assert_fits_in_machine_size(x):
     pass
@@ -121,7 +57,7 @@ def test_generates_and_minimizes():
 
 def test_can_minimize_large_arrays():
     x = minimal(
-        nps.arrays(u"uint32", 100),
+        nps.arrays("uint32", 100),
         lambda x: np.any(x) and not np.all(x),
         timeout_after=60,
     )
@@ -135,7 +71,7 @@ def test_can_minimize_float_arrays():
     assert x.sum() in (1, 50)
 
 
-class Foo(object):
+class Foo:
     pass
 
 
@@ -143,7 +79,7 @@ foos = st.tuples().map(lambda _: Foo())
 
 
 def test_can_create_arrays_of_composite_types():
-    arr = minimal(nps.arrays(object, 100, foos))
+    arr = minimal(nps.arrays(object, 100, elements=foos))
     for x in arr:
         assert isinstance(x, Foo)
 
@@ -158,18 +94,20 @@ def test_can_create_zero_dim_arrays_of_lists(x, data):
 
 def test_can_create_arrays_of_tuples():
     arr = minimal(
-        nps.arrays(object, 10, st.tuples(st.integers(), st.integers())),
+        nps.arrays(object, 10, elements=st.tuples(st.integers(), st.integers())),
         lambda x: all(t0 != t1 for t0, t1 in x),
     )
     assert all(a in ((1, 0), (0, 1)) for a in arr)
 
 
-@given(nps.arrays(object, (2, 2), st.tuples(st.integers())))
+@given(nps.arrays(object, (2, 2), elements=st.tuples(st.integers())))
 def test_does_not_flatten_arrays_of_tuples(arr):
     assert isinstance(arr[0][0], tuple)
 
 
-@given(nps.arrays(object, (2, 2), st.lists(st.integers(), min_size=1, max_size=1)))
+@given(
+    nps.arrays(object, (2, 2), elements=st.lists(st.integers(), min_size=1, max_size=1))
+)
 def test_does_not_flatten_arrays_of_lists(arr):
     assert isinstance(arr[0][0], list)
 
@@ -185,7 +123,10 @@ def test_can_generate_array_shapes(shape):
 def test_minimise_array_shapes(min_dims, dim_range, min_side, side_range):
     smallest = minimal(
         nps.array_shapes(
-            min_dims, min_dims + dim_range, min_side, min_side + side_range
+            min_dims=min_dims,
+            max_dims=min_dims + dim_range,
+            min_side=min_side,
+            max_side=min_side + side_range,
         )
     )
     assert len(smallest) == min_dims and all(k == min_side for k in smallest)
@@ -228,29 +169,17 @@ def test_can_generate_data_compound_dtypes(arr):
     assert isinstance(arr, np.ndarray)
 
 
-@settings(max_examples=100)
-@given(nps.nested_dtypes(max_itemsize=400), st.data())
-def test_infer_strategy_from_dtype(dtype, data):
-    # Given a dtype
-    assert isinstance(dtype, np.dtype)
-    # We can infer a strategy
-    strat = nps.from_dtype(dtype)
-    assert isinstance(strat, SearchStrategy)
-    # And use it to fill an array of that dtype
-    data.draw(nps.arrays(dtype, 10, strat))
-
-
 @given(nps.nested_dtypes())
 def test_np_dtype_is_idempotent(dtype):
     assert dtype == np.dtype(dtype)
 
 
 def test_minimise_scalar_dtypes():
-    assert minimal(nps.scalar_dtypes()) == np.dtype(u"bool")
+    assert minimal(nps.scalar_dtypes()) == np.dtype("bool")
 
 
 def test_minimise_nested_types():
-    assert minimal(nps.nested_dtypes()) == np.dtype(u"bool")
+    assert minimal(nps.nested_dtypes()) == np.dtype("bool")
 
 
 def test_minimise_array_strategy():
@@ -260,19 +189,23 @@ def test_minimise_array_strategy():
             nps.array_shapes(max_dims=3, max_side=3),
         )
     )
-    assert smallest.dtype == np.dtype(u"bool") and not smallest.any()
+    assert smallest.dtype == np.dtype("bool") and not smallest.any()
 
 
 @given(nps.array_dtypes(allow_subarrays=False))
 def test_can_turn_off_subarrays(dt):
-    for field, _ in dt.fields.values():
-        assert field.shape == ()
+    for name in dt.names:
+        assert dt.fields[name][0].shape == ()
+
+
+def test_array_dtypes_may_have_field_titles():
+    find_any(nps.array_dtypes(), lambda dt: len(dt.fields) > len(dt.names))
 
 
 @pytest.mark.parametrize("byteorder", ["<", ">"])
 @given(data=st.data())
 def test_can_restrict_endianness(data, byteorder):
-    dtype = data.draw(nps.integer_dtypes(byteorder, sizes=(16, 32, 64)))
+    dtype = data.draw(nps.integer_dtypes(endianness=byteorder, sizes=(16, 32, 64)))
     if byteorder == ("<" if sys.byteorder == "little" else ">"):
         assert dtype.byteorder == "="
     else:
@@ -292,21 +225,6 @@ def test_can_draw_arrays_from_scalars(data):
     assert result.dtype == dt
 
 
-@given(st.data())
-def test_can_cast_for_scalars(data):
-    # Note: this only passes with castable datatypes, certain dtype
-    # combinations will result in an error if numpy is not able to cast them.
-    dt_elements = np.dtype(data.draw(st.sampled_from(["bool", "<i2", ">i2"])))
-    dt_desired = np.dtype(
-        data.draw(st.sampled_from(["<i2", ">i2", "float32", "float64"]))
-    )
-    result = data.draw(
-        nps.arrays(dtype=dt_desired, elements=nps.from_dtype(dt_elements), shape=())
-    )
-    assert isinstance(result, np.ndarray)
-    assert result.dtype == dt_desired
-
-
 @given(st.data())
 def test_can_cast_for_arrays(data):
     # Note: this only passes with castable datatypes, certain dtype
@@ -324,26 +242,6 @@ def test_can_cast_for_arrays(data):
     assert result.dtype == dt_desired
 
 
-@given(st.data())
-def test_unicode_string_dtypes_generate_unicode_strings(data):
-    dt = data.draw(nps.unicode_string_dtypes())
-    result = data.draw(nps.from_dtype(dt))
-    assert isinstance(result, text_type)
-
-
-@given(st.data())
-def test_byte_string_dtypes_generate_unicode_strings(data):
-    dt = data.draw(nps.byte_string_dtypes())
-    result = data.draw(nps.from_dtype(dt))
-    assert isinstance(result, binary_type)
-
-
-@pytest.mark.parametrize("dtype", ["U", "S", "a"])
-def test_unsized_strings_length_gt_one(dtype):
-    # See https://github.com/HypothesisWorks/hypothesis/issues/2229
-    find_any(nps.arrays(dtype=dtype, shape=1), lambda arr: len(arr[0]) >= 2)
-
-
 @given(nps.arrays(dtype="int8", shape=st.integers(0, 20), unique=True))
 def test_array_values_are_unique(arr):
     assert len(set(arr)) == len(arr)
@@ -359,7 +257,7 @@ def test_cannot_generate_unique_array_of_too_many_elements():
     nps.arrays(
         elements=st.just(0.0),
         dtype=float,
-        fill=st.just(float("nan")),
+        fill=st.just(np.nan),
         shape=st.integers(0, 20),
         unique=True,
     )
@@ -381,7 +279,7 @@ def test_may_fill_with_nan_when_unique_is_set():
             elements=st.floats(allow_nan=False),
             shape=10,
             unique=True,
-            fill=st.just(float("nan")),
+            fill=st.just(np.nan),
         ),
         lambda x: np.isnan(x).any(),
     )
@@ -393,7 +291,7 @@ def test_may_fill_with_nan_when_unique_is_set():
         elements=st.floats(allow_nan=False),
         shape=10,
         unique=True,
-        fill=st.just(float("nan")),
+        fill=st.just(np.nan),
     )
 )
 def test_is_still_unique_with_nan_fill(xs):
@@ -415,48 +313,13 @@ def test_may_not_fill_with_non_nan_when_unique_is_set(arr):
 
 
 @fails_with(InvalidArgument)
-@given(nps.arrays(dtype="U", shape=10, unique=True, fill=st.just(u"")))
+@given(nps.arrays(dtype="U", shape=10, unique=True, fill=st.just("")))
 def test_may_not_fill_with_non_nan_when_unique_is_set_and_type_is_not_number(arr):
     pass
 
 
-@given(
-    st.data(),
-    st.builds(
-        "{}[{}]".format,
-        st.sampled_from(("datetime64", "timedelta64")),
-        st.sampled_from(nps.TIME_RESOLUTIONS),
-    ).map(np.dtype),
-)
-def test_inferring_from_time_dtypes_gives_same_dtype(data, dtype):
-    ex = data.draw(nps.from_dtype(dtype))
-    assert dtype == ex.dtype
-
-
-@given(st.data(), nps.byte_string_dtypes() | nps.unicode_string_dtypes())
-def test_inferred_string_strategies_roundtrip(data, dtype):
-    # Check that we never generate too-long or nul-terminated strings, which
-    # cannot be read back out of an array.
-    arr = np.zeros(shape=1, dtype=dtype)
-    ex = data.draw(nps.from_dtype(arr.dtype))
-    arr[0] = ex
-    assert arr[0] == ex
-
-
-@given(st.data(), nps.scalar_dtypes())
-def test_all_inferred_scalar_strategies_roundtrip(data, dtype):
-    # We only check scalars here, because record/compound/nested dtypes always
-    # give an array of np.void objects.  We're interested in whether scalar
-    # values are safe, not known type coercion.
-    arr = np.zeros(shape=1, dtype=dtype)
-    ex = data.draw(nps.from_dtype(arr.dtype))
-    assume(ex == ex)  # If not, the roundtrip test *should* fail!  (eg NaN)
-    arr[0] = ex
-    assert arr[0] == ex
-
-
 @pytest.mark.parametrize("fill", [False, True])
-@checks_deprecated_behaviour
+@fails_with(InvalidArgument)
 @given(st.data())
 def test_overflowing_integers_are_deprecated(fill, data):
     kw = {"elements": st.just(300)}
@@ -472,12 +335,15 @@ def test_overflowing_integers_are_deprecated(fill, data):
     [
         ("float16", st.floats(min_value=65520, allow_infinity=False)),
         ("float32", st.floats(min_value=10 ** 40, allow_infinity=False)),
-        ("complex64", st.complex_numbers(10 ** 300, allow_infinity=False)),
+        (
+            "complex64",
+            st.complex_numbers(min_magnitude=10 ** 300, allow_infinity=False),
+        ),
         ("U1", st.text(min_size=2, max_size=2)),
         ("S1", st.binary(min_size=2, max_size=2)),
     ],
 )
-@checks_deprecated_behaviour
+@fails_with(InvalidArgument)
 @given(data=st.data())
 def test_unrepresentable_elements_are_deprecated(fill, dtype, strat, data):
     if fill:
@@ -501,6 +367,29 @@ def test_inferred_floats_do_not_overflow(arr):
     pass
 
 
+@given(nps.arrays(dtype="float16", shape=10, elements={"min_value": 0, "max_value": 1}))
+def test_inferred_floats_can_be_constrained_at_low_width(arr):
+    assert (arr >= 0).all()
+    assert (arr <= 1).all()
+
+
+@given(
+    nps.arrays(
+        dtype="float16",
+        shape=10,
+        elements={
+            "min_value": 0,
+            "max_value": 1,
+            "exclude_min": True,
+            "exclude_max": True,
+        },
+    )
+)
+def test_inferred_floats_can_be_constrained_at_low_width_excluding_endpoints(arr):
+    assert (arr > 0).all()
+    assert (arr < 1).all()
+
+
 @given(
     nps.arrays(
         dtype="float16",
@@ -526,7 +415,9 @@ def test_unique_array_without_fill(arr):
 def test_mapped_positive_axes_are_unique(ndim, data):
     min_size = data.draw(st.integers(0, ndim), label="min_size")
     max_size = data.draw(st.integers(min_size, ndim), label="max_size")
-    axes = data.draw(nps.valid_tuple_axes(ndim, min_size, max_size), label="axes")
+    axes = data.draw(
+        nps.valid_tuple_axes(ndim, min_size=min_size, max_size=max_size), label="axes"
+    )
     assert len(set(axes)) == len({i if 0 < i else ndim + i for i in axes})
 
 
@@ -534,7 +425,9 @@ def test_mapped_positive_axes_are_unique(ndim, data):
 def test_length_bounds_are_satisfied(ndim, data):
     min_size = data.draw(st.integers(0, ndim), label="min_size")
     max_size = data.draw(st.integers(min_size, ndim), label="max_size")
-    axes = data.draw(nps.valid_tuple_axes(ndim, min_size, max_size), label="axes")
+    axes = data.draw(
+        nps.valid_tuple_axes(ndim, min_size=min_size, max_size=max_size), label="axes"
+    )
     assert min_size <= len(axes) <= max_size
 
 
@@ -550,7 +443,7 @@ def test_axes_are_valid_inputs_to_sum(shape, data):
 def test_minimize_tuple_axes(ndim, data):
     min_size = data.draw(st.integers(0, ndim), label="min_size")
     max_size = data.draw(st.integers(min_size, ndim), label="max_size")
-    smallest = minimal(nps.valid_tuple_axes(ndim, min_size, max_size))
+    smallest = minimal(nps.valid_tuple_axes(ndim, min_size=min_size, max_size=max_size))
     assert len(smallest) == min_size and all(k > -1 for k in smallest)
 
 
@@ -560,7 +453,8 @@ def test_minimize_negative_tuple_axes(ndim, data):
     min_size = data.draw(st.integers(0, ndim), label="min_size")
     max_size = data.draw(st.integers(min_size, ndim), label="max_size")
     smallest = minimal(
-        nps.valid_tuple_axes(ndim, min_size, max_size), lambda x: all(i < 0 for i in x)
+        nps.valid_tuple_axes(ndim, min_size=min_size, max_size=max_size),
+        lambda x: all(i < 0 for i in x),
     )
     assert len(smallest) == min_size
 
@@ -668,8 +562,7 @@ def _draw_valid_bounds(data, shape, max_dims, permit_none=True):
     return min_side, max_side
 
 
-def _broadcast_two_shapes(shape_a, shape_b):
-    # type: (nps.Shape, nps.Shape) -> nps.Shape
+def _broadcast_two_shapes(shape_a: nps.Shape, shape_b: nps.Shape) -> nps.Shape:
     result = []
     for a, b in zip_longest(reversed(shape_a), reversed(shape_b), fillvalue=1):
         if a != b and (a != 1) and (b != 1):
@@ -803,7 +696,7 @@ def test_minimize_broadcastable_shape(min_dims, shape, data):
             max_dims=max_dims,
         )
     )
-    note("(smallest): {}".format(smallest))
+    note(f"(smallest): {smallest}")
     n_leading = max(len(smallest) - len(shape), 0)
     n_aligned = max(len(smallest) - n_leading, 0)
     expected = [min_side] * n_leading + [
@@ -1048,7 +941,7 @@ def test_mutually_broadcastable_shapes_can_generate_arbitrary_ndims(
             base_shape=base_shape,
             min_side=0,
             max_dims=max_dims,
-            **kwargs
+            **kwargs,
         ),
         lambda x: {len(s) for s in x.input_shapes} == set(desired_ndims),
         settings(max_examples=10 ** 6),
@@ -1173,6 +1066,13 @@ def test_basic_indices_can_generate_empty_tuple():
     find_any(nps.basic_indices(shape=(0, 0), allow_ellipsis=True), lambda ix: ix == ())
 
 
+def test_basic_indices_can_generate_non_tuples():
+    find_any(
+        nps.basic_indices(shape=(0, 0), allow_ellipsis=True),
+        lambda ix: not isinstance(ix, tuple),
+    )
+
+
 def test_basic_indices_can_generate_long_ellipsis():
     # Runs of slice(None) - such as [0,:,:,:,0] - can be replaced by e.g. [0,...,0]
     find_any(
@@ -1181,7 +1081,11 @@ def test_basic_indices_can_generate_long_ellipsis():
     )
 
 
-@given(nps.basic_indices(shape=(0, 0, 0, 0, 0)).filter(lambda idx: Ellipsis in idx))
+@given(
+    nps.basic_indices(shape=(0, 0, 0, 0, 0)).filter(
+        lambda idx: isinstance(idx, tuple) and Ellipsis in idx
+    )
+)
 def test_basic_indices_replaces_whole_axis_slices_with_ellipsis(idx):
     # If ... is in the slice, it replaces all ,:, entries for this shape.
     assert slice(None) not in idx
@@ -1211,7 +1115,10 @@ def test_basic_indices_generate_valid_indexers(
     )
     # Check that disallowed things are indeed absent
     if not allow_newaxis:
-        assert 0 <= len(indexer) <= len(shape) + int(allow_ellipsis)
+        if isinstance(indexer, tuple):
+            assert 0 <= len(indexer) <= len(shape) + int(allow_ellipsis)
+        else:
+            assert 1 <= len(shape) + int(allow_ellipsis)
         assert np.newaxis not in shape
     if not allow_ellipsis:
         assert Ellipsis not in shape
@@ -1231,3 +1138,14 @@ def test_basic_indices_generate_valid_indexers(
         assert min_dims <= view.ndim <= (32 if max_dims is None else max_dims)
         if view.size:
             assert np.shares_memory(view, array)
+
+
+# addresses https://github.com/HypothesisWorks/hypothesis/issues/2582
+@given(
+    nps.arrays(
+        shape=nps.array_shapes(min_dims=0, min_side=0), dtype=nps.floating_dtypes()
+    )
+)
+def test_array_owns_memory(x: np.ndarray):
+    assert x.base is None
+    assert x[...].base is x
